/*
    IR Messenger - Infrared chat on TI89 calculators
    Copyright (C) 2003-2004  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _NO_INCLUDE_PATCH

#include "config.h"

#include <compat.h>
#include <graph.h>
#include <kbd.h>
#include <statline.h>
#include <system.h>

#include "chatroom.h"
#include "icon.h"
#include "icon_chat.h"
#include "icon_config.h"
#include "icon_minimize.h"
#include "locale.h"
#include "settings.h"

void *kbq;
unsigned char current_sel;

unsigned short getkey()
{
  unsigned short key;
  while(OSdequeue(&key, kbq)) idle();
  return key;
}

unsigned char screen_on;
unsigned char chatmode;

#define XOR1 ScrRectFill(&(SCR_RECT){{21, 39, 139, 49}}, ScrRect, A_XOR)
#define XOR2 ScrRectFill(&(SCR_RECT){{21, 55, 139, 65}}, ScrRect, A_XOR)
#define XOR3 ScrRectFill(&(SCR_RECT){{21, 71, 139, 81}}, ScrRect, A_XOR)

void rounded_rect(short x, short y, short x2, short y2)
{
  DrawLine(x+1, y, x2-1, y, A_NORMAL);
  DrawLine(x+1, y2, x2-1, y2, A_NORMAL);
  DrawLine(x, y+1, x, y2-1, A_NORMAL);
  DrawLine(x2, y+1, x2, y2-1, A_NORMAL);
}

unsigned char current_sel;

void draw_screen()
{
  ClrScr();

  BitmapPut(0, 0, &icon, ScrRect, A_NORMAL);
  FontSetSys(F_8x10);
  DrawStr(33, 0, "IR Messenger", A_NORMAL);
  FontSetSys(F_6x8);
  DrawStr(33, 11, "Version "VERSION, A_NORMAL);
  FontSetSys(F_4x6);
  DrawStr(33, 22, "fourmiz.org/~daseb/ti89inside", A_NORMAL);
  DrawLine(0, 32, 160, 32, A_NORMAL);

  rounded_rect(20, 38, 140, 50);
  BitmapPut(23, 39, &icon_chat, ScrRect, A_NORMAL);
  DrawStr(40, 42, START_CHATTING, A_NORMAL);

  rounded_rect(20, 54, 140, 66);
  BitmapPut(23, 55, &icon_config, ScrRect, A_NORMAL);
  DrawStr(40, 58, SETTINGS_MENU, A_NORMAL);

  rounded_rect(20, 70, 140, 82);
  BitmapPut(23, 71, &icon_minimize, ScrRect, A_NORMAL);
  DrawStr(40, 74, MINIMIZE, A_NORMAL);

  current_sel = 1;
  XOR1;
  DrawLine(0, LCD_HEIGHT-7, LCD_WIDTH, LCD_HEIGHT-7, A_NORMAL);
}

LCD_BUFFER VirtualScreen, ScreenBackup;

void switch_screen()
{
  unsigned short exit;

  if(screen_on) return;
  screen_on = TRUE;

  LCD_save(ScreenBackup);
  LCD_restore(VirtualScreen);
  ST_busy(ST_NORMAL);

  exit = FALSE;

  if(chatmode) {
    chatroom_start();
    if(chatmode) exit = TRUE; else { draw_screen(); ST_helpMsg(START_CHATTING_HELP); }
  }

  if(current_sel == 1) ST_helpMsg(START_CHATTING_HELP);
  if(current_sel == 2) ST_helpMsg(SETTINGS_HELP);
  if(current_sel == 3) ST_helpMsg(MINIMIZE_HELP);
  while(!exit) {
    switch(getkey()) {
      case KEY_UP:
        if(current_sel != 1) {
          if(current_sel == 2) XOR2;
          if(current_sel == 3) XOR3;
          current_sel--;
          if(current_sel == 1) { XOR1; ST_helpMsg(START_CHATTING_HELP); };
          if(current_sel == 2) { XOR2; ST_helpMsg(SETTINGS_HELP); }
        }
        break;
      case KEY_DOWN:
        if(current_sel != 3) {
          if(current_sel == 1) XOR1;
          if(current_sel == 2) XOR2;
          current_sel++;
          if(current_sel == 2) { XOR2; ST_helpMsg(SETTINGS_HELP); }
          if(current_sel == 3) { XOR3; ST_helpMsg(MINIMIZE_HELP); }
        }
        break;
      case KEY_ENTER:
        switch(current_sel) {
          case 1:
            chatroom_start();
            if(chatmode) exit = TRUE; else { draw_screen(); ST_helpMsg(START_CHATTING_HELP); }
            break;
          case 2:
            show_config_dialog();
            ST_helpMsg(SETTINGS_HELP);
            ST_busy(ST_NORMAL);
            break;
          case 3:
            exit = TRUE;
            break;
        }
        break;
      case HOTKEY:
      case KEY_ESC:
        exit = TRUE;
        break;
    }
  }

  LCD_save(VirtualScreen);
  LCD_restore(ScreenBackup);
  ST_eraseHelp();
  ST_busy(ST_NORMAL);
  ST_busy(ST_IDLE);
  screen_on = FALSE;
}

void init_screen()
{
  PortSet(VirtualScreen, 239, 127);
  draw_screen();
  PortRestore();
  chatmode = FALSE;
  screen_on = FALSE;
  kbq = kbd_queue();
}

